#include "stdafx.h"
#include "fmod_dsp2.h"
#include "fmod.h"
#include "SoundTouch.h"

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}

/*
	*** VARIOUS ***
*/
const float PI =  3.141592653589793f;

BOOL b_PitchScale[6]	= {TRUE,FALSE,FALSE,FALSE, FALSE, FALSE};

const int DSP_PITCHSCALE	= 1;

FSOUND_STREAM *src_stream[6];
BOOL src_mono[6];

/*
	*** PITCHSCALE ***
*/
SoundTouch SoundTouch;

short	pitch_BufOut[6][40960];
short	pitch_BufOutTmp[2048];
int		pitch_BufOutPosWrite[6] = {0,0,0,0,0,0};
int		pitch_BufOutPosRead[6] = {0,0,0,0,0,0};

/*
	*** EFFECTS ***
 Main Function
*/
void *DSP_EffectCallback(void *originalbuffer, void *newbuffer, int length, int param)
{
	signed short	*srcleft, *srcright;
	//float			*srcleft2, *srcright2;

	int l, r;
	int nSamples=0;
	int bla = 0;
	int i = 0;

	//Checking for mixertype is only necessary when using this callback in a
	//dsp on the whole output stream
	//It is not necessary with Stream DSP's
	//if (FSOUND_GetMixer() == FSOUND_MIXER_QUALITY_FPU) {
	//	srcleft2 = ((float *)newbuffer);
	//	srcright2= ((float *)newbuffer)+1;
	//} else {
	srcleft   = ((signed short *)newbuffer);
	srcright  = ((signed short *)newbuffer)+1;
	//}

	if (!src_mono[param])
		length = length * 2;	// /2 for mono

	//Process
	if (b_PitchScale[param]) {
		SoundTouch.putSamples(srcleft, length / 2);
		do {
			// receive processed audio data
			nSamples = SoundTouch.receiveSamples(pitch_BufOutTmp, 1024);
			// calculate processed audio data size in bytes
			for (i = 0; i < nSamples*2; i+=2)
			{
				pitch_BufOut[param][pitch_BufOutPosWrite[param]++] = pitch_BufOutTmp[i];
				pitch_BufOut[param][pitch_BufOutPosWrite[param]++] = pitch_BufOutTmp[i+1];
				if (pitch_BufOutPosWrite[param] >= 40960)
					pitch_BufOutPosWrite[param] = 0;
			}
		} while (nSamples != 0);
	}

	//*** MAIN LOOP ***
	for (int count = 0; count < length; count += 2)
	{
		//if (FSOUND_GetMixer() == FSOUND_MIXER_QUALITY_FPU) {
		//l = (signed short)srcleft2[count];
		//r = (signed short)srcright2[count];
		//} else {
		l = srcleft[count];
		r = srcright[count];
		//}
		
		//*** PitchScale
		if (b_PitchScale[param])
		{
			if (pitch_BufOutPosWrite[param] != pitch_BufOutPosRead[param])
			{
				l = pitch_BufOut[param][pitch_BufOutPosRead[param]++];
				r = pitch_BufOut[param][pitch_BufOutPosRead[param]++];
				if (pitch_BufOutPosRead[param] == 40960)
					pitch_BufOutPosRead[param] = 0;
			} else {
				l = 0;
				r = 0;
			}
		}

		//*** GENERAL ***
		//Clip and write new values
		if      (l < -32768) l = -32768;
		else if (l >  32767) l =  32767;
		if      (r < -32768) r = -32768;
		else if (r >  32767) r =  32767;

		srcleft[count] = l;
		srcright[count]= r;
	}

	return newbuffer;
}

FSOUND_DSPUNIT *dsp_effect_create(FSOUND_STREAM *stream, int priority, int nr, FSOUND_STREAM *h_stream)
{
	if ((nr < 0) || (nr > 5))
		return 0;

	//*** MONO/STEREO ***
	src_stream[nr]   = h_stream;
	FSOUND_SAMPLE *tmp_sample = FSOUND_Stream_GetSample(h_stream);
	unsigned int tmp_mode = FSOUND_Sample_GetMode(tmp_sample);
	src_mono[nr] = (tmp_mode & 0x00000020);

	//*** SOUNDPITCH ***
	SoundTouch.setSampleRate(44100);
	SoundTouch.setChannels(2);

    SoundTouch.setTempo(1.0f);
    SoundTouch.setRate(1.0f);
    SoundTouch.setPitch(1.0f);

	//*** DSP Creation ***
	FSOUND_DSPUNIT *dsp1;
	dsp1 = FSOUND_Stream_CreateDSP(stream, &DSP_EffectCallback, priority, nr);

	return dsp1;
}

int dsp_effect_enable(int nr, int effect, int enable)
{
	if (nr > 5)
		return -1;

	BOOL tmpBool;
	if (enable == 0)
		tmpBool=FALSE;
	else
		tmpBool=TRUE;

	if (effect == DSP_PITCHSCALE)
		b_PitchScale[nr]   = tmpBool;

	return 0;
}

int dsp_pitch_update(int nr, float tempo, float rate, float pitch)
{
    SoundTouch.setTempo(tempo);
    SoundTouch.setRate(rate);
    SoundTouch.setPitch(pitch);

	return 1;
}